@@ -23,7 +23,7 @@
 #include "sebie_view.h"
 #include "sebie_gtk_helpers.h"
 #include "sebie_config.h"
-#include "../config.h"
+#include "config.h"
 
 #include <gdk/gdkkeysyms.h>
 #include <stdlib.h>
@@ -40,7 +40,7 @@ sebie_update_view(struct sebie_s *sebie)
     snprintf(buf, 6, "%5.2f", sebie->model.selection_fac);
     gtk_label_set_text(GTK_LABEL(sebie->view.label_selection_fac), buf);
   }
-  
+
   if (sebie->dirty_tags & SEBIE_UPDATE_SELECTION_FAC_W) {
     gchar buf[6];
     snprintf(buf, 6, "%d", sebie->model.selection_fac_width);
@@ -63,34 +63,60 @@ sebie_update_view(struct sebie_s *sebie)
       sebie->dirty_tags & SEBIE_UPDATE_SELECTION_RECT) {
     /* do it the rough way, always draw the whole area and not just the
      * areas which changed  */
+#if GTK_CHECK_VERSION(3,0,0)
+#if GTK_CHECK_VERSION(3,22,0)
+    GdkWindow *window = gtk_widget_get_window(sebie->view.drawingarea1);
+    cairo_region_t *cairo_region = cairo_region_create();
+    GdkDrawingContext *drawing_context = gdk_window_begin_draw_frame (window, cairo_region);
+    cairo_t *cr = gdk_drawing_context_get_cairo_context(drawing_context);
+#else
+    GdkWindow *window = gtk_widget_get_window(sebie->view.drawingarea1);
+    cairo_t *cr = gdk_cairo_create (window);
+#endif
+#else
     GdkGC *gdkGC = gdk_gc_new(sebie->view.drawingarea1->window);
-    
-    if (sebie->dirty_tags & SEBIE_UPDATE_SCALED_PIXBUF) {
+#endif
+
+    // SEBIE_UPDATE_SELECTION_RECT could be optimized to (SEBIE_UPDATE_SELECTION_RECT && rect extens scaled_pixbuf area)
+    if ((sebie->dirty_tags & SEBIE_UPDATE_SCALED_PIXBUF) || (sebie->dirty_tags & SEBIE_UPDATE_SELECTION_RECT)) {
+#if GTK_CHECK_VERSION(3,0,0)
+      int drawingarea1_width = gtk_widget_get_allocated_width(sebie->view.drawingarea1);
+      int drawingarea2_height = gtk_widget_get_allocated_height(sebie->view.drawingarea1);
+      cairo_set_source_rgb(cr, 0.878, 0.878, 0.878);
+      cairo_rectangle (cr, 0, 0, drawingarea1_width, drawingarea2_height);
+      cairo_fill(cr);
+#else
       GdkColor color;
       color.red   = 0xe0e0;
       color.green = 0xe0e0;
       color.blue  = 0xe0e0;
       gdk_gc_set_rgb_fg_color(gdkGC, &color);
       gdk_draw_rectangle(sebie->view.drawingarea1->window,
-			 gdkGC,
-			 TRUE,
-			 0, 0,
-			 -1, -1);
+                         gdkGC,
+                         TRUE,
+                         0, 0,
+                         -1, -1);
+#endif
     }
-    
+
     if (sebie->model.scaled_pixbuf != NULL) {
+#if GTK_CHECK_VERSION(3,0,0)
+      gdk_cairo_set_source_pixbuf (cr, sebie->model.scaled_pixbuf, 0, 0);
+      cairo_paint (cr);
+#else
       gdk_pixbuf_render_to_drawable (sebie->model.scaled_pixbuf,
-				     sebie->view.drawingarea1->window,
-				     gdkGC,
-				     0, 0,
-				     0, 0,
-				     sebie->model.scaled_width,
-				     sebie->model.scaled_height,
-				     GDK_RGB_DITHER_NONE,
-				     0,
-				     0);
+                                     sebie->view.drawingarea1->window,
+                                     gdkGC,
+                                     0, 0,
+                                     0, 0,
+                                     sebie->model.scaled_width,
+                                     sebie->model.scaled_height,
+                                     GDK_RGB_DITHER_NONE,
+                                     0,
+                                     0);
+#endif
     }
-    
+
     gint rx, ry, rheight, rwidth;
 
     if (sebie->model.selection_width > 0) {
@@ -100,7 +126,7 @@ sebie_update_view(struct sebie_s *sebie)
       rx = sebie->model.selection_x + sebie->model.selection_width;
       rwidth = - sebie->model.selection_width;
     }
-    
+
     if (sebie->model.selection_height > 0) {
       ry = sebie->model.selection_y;
       rheight = sebie->model.selection_height;
@@ -108,33 +134,50 @@ sebie_update_view(struct sebie_s *sebie)
       ry = sebie->model.selection_y + sebie->model.selection_height;
       rheight = - sebie->model.selection_height;
     }
-
+#if GTK_CHECK_VERSION(3,0,0)
+    cairo_set_source_rgb (cr, 1., 1., 1.);
+    cairo_set_operator (cr, CAIRO_OPERATOR_DIFFERENCE);
+    cairo_rectangle (cr, rx, ry, rwidth, rheight);
+    cairo_set_line_width(cr, 1);
+    cairo_stroke(cr);
+#else
     gdk_gc_set_function(gdkGC, GDK_INVERT);
     gdk_draw_rectangle (sebie->view.drawingarea1->window,
-			gdkGC,
-			FALSE,
-			rx,
-			ry,
-			rwidth,
-			rheight);
+                        gdkGC,
+                        FALSE,
+                        rx,
+                        ry,
+                        rwidth,
+                        rheight);
+#endif
+
+#if GTK_CHECK_VERSION(3,0,0)
+#if GTK_CHECK_VERSION(3,22,0)
+    gdk_window_end_draw_frame(window,drawing_context);
+    cairo_region_destroy(cairo_region);
+#else
+    cairo_paint (cr);
+    cairo_destroy (cr);
+#endif
+#endif
   }
 
   if (sebie->dirty_tags & SEBIE_UPDATE_INPUT_PATH) {
     gtk_entry_set_text(GTK_ENTRY(sebie->view.entry_input_path),
-		       sebie->model.input_path);
+                       sebie->model.input_path);
   }
   if (sebie->dirty_tags & SEBIE_UPDATE_INPUT_REGEX) {
     gtk_entry_set_text(GTK_ENTRY(sebie->view.entry_input_regex),
-		       sebie->model.input_regex);
+                       sebie->model.input_regex);
   }
   if (sebie->dirty_tags & SEBIE_UPDATE_OUTPUT_FORMAT) {
     gtk_entry_set_text(GTK_ENTRY(sebie->view.entry_output_format),
-		       sebie->model.output_format);
+                       sebie->model.output_format);
   }
-  
+
   if (sebie->dirty_tags & SEBIE_UPDATE_OUTPUT_FILE_NAME) {
     gtk_entry_set_text(GTK_ENTRY(sebie->view.entry_output_file),
-		       sebie->model.output_file_name);
+                       sebie->model.output_file_name);
   }
 
   if (sebie->dirty_tags & SEBIE_UPDATE_STATUS_BAR) {
@@ -151,11 +194,11 @@ void
 sebie_error_popup(struct sebie_s *sebie, const gchar *error_str)
 {
   GtkWidget *dialog = gtk_message_dialog_new (GTK_WINDOW(sebie->view.window1),
-					      GTK_DIALOG_DESTROY_WITH_PARENT,
-					      GTK_MESSAGE_ERROR,
-					      GTK_BUTTONS_CLOSE,
-                          "%s",
-                          error_str);
+                                              GTK_DIALOG_DESTROY_WITH_PARENT,
+                                              GTK_MESSAGE_ERROR,
+                                              GTK_BUTTONS_CLOSE,
+                                              "%s",
+                                              error_str);
 #if 1
   // modal dialog: you must click to resume
   gtk_dialog_run (GTK_DIALOG (dialog));
@@ -163,8 +206,8 @@ sebie_error_popup(struct sebie_s *sebie, const gchar *error_str)
 #else
   // non modal dialog: application will run further
   g_signal_connect_swapped (GTK_OBJECT (dialog), "response",
-			    G_CALLBACK (gtk_widget_destroy),
-			    GTK_OBJECT (dialog));
+                            G_CALLBACK (gtk_widget_destroy),
+                            GTK_OBJECT (dialog));
   gtk_widget_show (dialog);
 #endif
 }
@@ -198,7 +241,7 @@ load_pixbuf (struct sebie_s *sebie)
   if (sebie->model.input_file_name == NULL) {
     return;
   }
-  
+
   GError *error = NULL;
   GdkPixbuf *pixbuf;
 
@@ -223,7 +266,7 @@ load_pixbuf (struct sebie_s *sebie)
   double scale2 = (double)sebie->model.pixbuf_height / (double)sebie->model.image_width;
   double scale3 = (double)sebie->model.pixbuf_width / (double)sebie->model.image_height;
   double scale4 = (double)sebie->model.pixbuf_height / (double)sebie->model.image_height;
-    
+
   scale1 = MIN(scale1, scale2);
   scale1 = MIN(scale1, scale3);
   scale1 = MIN(scale1, scale4);
@@ -234,8 +277,8 @@ load_pixbuf (struct sebie_s *sebie)
   sebie->model.scaled_rotation = SEBIE_ROTATE_000;
 
   sebie_debug("scaled: width: %d height: %d \n",
-	      sebie->model.scaled_width, 
-	      sebie->model.scaled_height);
+              sebie->model.scaled_width,
+              sebie->model.scaled_height);
   if (sebie->model.scaled_pixbuf_orig != NULL) {
     g_object_unref(sebie->model.scaled_pixbuf_orig);
   }
@@ -243,12 +286,12 @@ load_pixbuf (struct sebie_s *sebie)
     g_object_unref(sebie->model.scaled_pixbuf);
   }
   sebie->model.scaled_pixbuf_orig = gdk_pixbuf_scale_simple(sebie->model.image_pixbuf,
-							    sebie->model.scaled_width,
-							    sebie->model.scaled_height,
-							    GDK_INTERP_HYPER);
+                                                            sebie->model.scaled_width,
+                                                            sebie->model.scaled_height,
+                                                            GDK_INTERP_HYPER);
   sebie->model.scaled_pixbuf = pixbuf_color_corrections(sebie->model.scaled_pixbuf_orig,
-							sebie->model.image_gamma,
-							sebie->model.image_grayscale);
+                                                        sebie->model.image_gamma,
+                                                        sebie->model.image_grayscale);
   sebie->dirty_tags |= SEBIE_UPDATE_SCALED_PIXBUF;
   sebie->dirty_tags |= SEBIE_UPDATE_SELECTION_RECT;
 #if 0
@@ -274,7 +317,7 @@ sebie_update_file_list(struct sebie_s *sebie)
 
     if (sebie->model.regex_status) {
       if (regexec(&sebie->model.regex, next, 0, NULL, 0) == 0) {
-	sebie_file_list_add(sebie, next);
+          sebie_file_list_add(sebie, next);
       }
     } else {
       sebie_debug("sebie->model.regex_status invalid");
@@ -289,14 +332,14 @@ rotate_scaled(struct sebie_s *sebie, int direction)
   if (direction == SEBIE_ROTATE_090 ||
       direction == SEBIE_ROTATE_270 ) {
     GdkPixbuf *rotated = rotate_pixbuf(sebie->model.scaled_pixbuf, direction);
-    
+
     sebie->model.scaled_width = gdk_pixbuf_get_width(rotated);
     sebie->model.scaled_height = gdk_pixbuf_get_height(rotated);
     if (sebie->model.scaled_pixbuf != NULL) {
       g_object_unref(sebie->model.scaled_pixbuf);
     }
     sebie->model.scaled_pixbuf = rotated;
-  
+
     sebie->dirty_tags |= SEBIE_UPDATE_SCALED_PIXBUF;
     sebie->dirty_tags |= SEBIE_UPDATE_SELECTION_RECT;
     sebie_update_view(sebie);
@@ -310,20 +353,20 @@ apply_color_corrections(struct sebie_s *sebie)
   if (sebie->model.scaled_pixbuf_orig != NULL &&
       sebie->model.scaled_pixbuf != NULL) {
     GdkPixbuf *corr = pixbuf_color_corrections(sebie->model.scaled_pixbuf_orig,
-					       sebie->model.image_gamma,
-					       sebie->model.image_grayscale);
+                                               sebie->model.image_gamma,
+                                               sebie->model.image_grayscale);
     g_object_unref(sebie->model.scaled_pixbuf);
-    
+
     if (sebie->model.scaled_rotation == SEBIE_ROTATE_000) {
       sebie->model.scaled_pixbuf = corr;
     } else {
       GdkPixbuf *rotated = rotate_pixbuf(corr, sebie->model.scaled_rotation);
       if (corr != NULL) {
-	g_object_unref(corr);
+        g_object_unref(corr);
       }
       sebie->model.scaled_pixbuf = rotated;
     }
-    
+
     sebie->dirty_tags |= SEBIE_UPDATE_SCALED_PIXBUF;
     sebie->dirty_tags |= SEBIE_UPDATE_SELECTION_RECT;
   }
@@ -362,14 +405,14 @@ init_sebie_model(struct sebie_s *sebie)
 {
   char *homedir;
   char buf[PATH_MAX];
-  
+
   if ((homedir = getenv("HOME")) != NULL) {
     snprintf(buf, PATH_MAX, "%s/%s", homedir, ".sebieconf");
     sebie->config_file = g_strdup(buf);
   } else {
     sebie->config_file = g_strdup(".sebieconf");
   }
-  
+
   sebie_config_spec config_spec[] = {
     { .name = "InputPath", .type = CONFIG_STRING, .storage.string = &sebie->model.input_path, .defvalue.string = "" },
     { .name = "InputRegEx", .type = CONFIG_STRING, .storage.string = &sebie->model.input_regex, .defvalue.string = "\\([0-9]*\\).jpg" },
@@ -397,16 +440,16 @@ init_sebie_model(struct sebie_s *sebie)
   sebie->dirty_tags |= SEBIE_UPDATE_OUTPUT_FORMAT;
 
   sebie->model.file_list = gtk_list_store_new(1, G_TYPE_STRING);
-  
+
   sebie->model.selection_fac = sebie->model.selection_fac_width / sebie->model.selection_fac_height;
 
   sebie->model.mouse_click_type = SEBIE_NOT_CLICKED;
-  
+
   sebie->dirty_tags |= SEBIE_UPDATE_SELECTION_FAC;
   sebie->dirty_tags |= SEBIE_UPDATE_SELECTION_FAC_W;
   sebie->dirty_tags |= SEBIE_UPDATE_SELECTION_FAC_H;
   sebie->dirty_tags |= SEBIE_UPDATE_IMAGE_GAMMA;
-  
+
   sebie->model.statusbar_text = strdup("   SeBIE-" PACKAGE_VERSION " (C) 2002-2010 by Peter.Seiderer@ciselant.de");
   sebie->dirty_tags |= SEBIE_UPDATE_STATUS_BAR;
 
@@ -418,13 +461,16 @@ int main(int argc, char *argv[])
 {
   GtkWidget *window1;
   struct sebie_s *sebie;
-  
+
   sebie = g_new0(struct sebie_s, 1);
 
+#if GTK_CHECK_VERSION(3,0,0)
+#else
   gtk_set_locale ();
+#endif
 
   gtk_init (&argc, &argv);
-  
+
   init_sebie_model(sebie);
   window1 = init_sebie_view(sebie);
   sebie_update_view(sebie);
