@@ -61,7 +61,7 @@ typedef enum sebie_model_tag_e {
 
 typedef enum sebie_mouse_click_e {
   SEBIE_NOT_CLICKED      = 0,
-  SEBIE_RESIZE_SELECTION = 1, 
+  SEBIE_RESIZE_SELECTION = 1,
   SEBIE_MOVE_SELECTION   = 2
 } sebie_mouse_click_type;
 
@@ -77,20 +77,36 @@ typedef struct sebie_view_s {
   GtkWidget *vbox1;
   GtkWidget *toolbar1;
   GtkWidget *tmp_toolbar_icon;
+#if GTK_CHECK_VERSION(3,0,0)
+  GtkToolItem *button_quit;
+  //GtkWidget *button_load;
+  GtkToolItem *button_save;
+  GtkToolItem *button_rotate_left;
+  GtkToolItem *button_rotate_right;
+#else
   GtkWidget *button_quit;
   //GtkWidget *button_load;
   GtkWidget *button_save;
   GtkWidget *button_rotate_left;
   GtkWidget *button_rotate_right;
+#endif
   GtkWidget *label_selection_fac_width;
   GtkWidget *entry_selection_fac_width;
   GtkWidget *label_selection_fac_height;
   GtkWidget *entry_selection_fac_height;
+#if GTK_CHECK_VERSION(3,0,0)
+  GtkToolItem *button_fac_swap;
+#else
   GtkWidget *button_fac_swap;
+#endif
   GtkWidget *label_selection_fac;
   GtkWidget *label_selection_gamma;
   GtkWidget *entry_selection_gamma;
+#if GTK_CHECK_VERSION(3,0,0)
+  GtkToolItem *button_grayscale;
+#else
   GtkWidget *button_grayscale;
+#endif
   GtkWidget *hpaned1;
   GtkWidget *vbox2;
   GtkWidget *label_input_path;
