@@ -31,42 +31,42 @@
 
 void
 on_treeview_file_list_changed(GtkTreeSelection *selection,
-			      gpointer user_data)
+                              gpointer user_data)
 {
-  struct sebie_s *sebie = (struct sebie_s *)user_data;
-  GtkTreeIter iter;
-  GtkTreeModel *model;
-  gchar *filename;
-  if (gtk_tree_selection_get_selected(selection, &model, &iter)) {
-    gtk_tree_model_get(model, &iter, 0, &filename, -1);
-    sebie_set_string(&sebie->model.input_file_name, filename);
-    load_pixbuf(sebie);
-    if (sebie->model.regex_status) {
-      int nmatch = 2;
-      regmatch_t matchptr[nmatch];
-      if (regexec(&sebie->model.regex, sebie->model.input_file_name, nmatch, matchptr, 0) == 0) {
-	char buf_regex[nmatch][256];
-	int i;
-	for (i = 0; i < nmatch; i++) {
-	  strncpy(buf_regex[i],sebie->model.input_file_name +  matchptr[i].rm_so, matchptr[i].rm_eo - matchptr[i].rm_so);
-	  buf_regex[i][matchptr[i].rm_eo - matchptr[i].rm_so] = '\0';
-	  sebie_debug("buf_regex[%d] = %s\n", i, buf_regex[i]);
-	}
-	char buf[256];
-	snprintf(buf, 256, sebie->model.output_format, buf_regex[1]);
-	sebie_set_string(&sebie->model.output_file_name, buf);
-	sebie->dirty_tags |= SEBIE_UPDATE_OUTPUT_FILE_NAME;
-	sebie_update_view(sebie);
+    struct sebie_s *sebie = (struct sebie_s *)user_data;
+    GtkTreeIter iter;
+    GtkTreeModel *model;
+    gchar *filename;
+    if (gtk_tree_selection_get_selected(selection, &model, &iter)) {
+        gtk_tree_model_get(model, &iter, 0, &filename, -1);
+        sebie_set_string(&sebie->model.input_file_name, filename);
+        load_pixbuf(sebie);
+        if (sebie->model.regex_status) {
+            int nmatch = 2;
+            regmatch_t matchptr[nmatch];
+            if (regexec(&sebie->model.regex, sebie->model.input_file_name, nmatch, matchptr, 0) == 0) {
+                char buf_regex[nmatch][256];
+                int i;
+                for (i = 0; i < nmatch; i++) {
+                    strncpy(buf_regex[i],sebie->model.input_file_name +  matchptr[i].rm_so, matchptr[i].rm_eo - matchptr[i].rm_so);
+                    buf_regex[i][matchptr[i].rm_eo - matchptr[i].rm_so] = '\0';
+                    sebie_debug("buf_regex[%d] = %s\n", i, buf_regex[i]);
+                }
+              char buf[256];
+              snprintf(buf, 256, sebie->model.output_format, buf_regex[1]);
+              sebie_set_string(&sebie->model.output_file_name, buf);
+              sebie->dirty_tags |= SEBIE_UPDATE_OUTPUT_FILE_NAME;
+              sebie_update_view(sebie);
+          }
+      } else {
+          sebie_debug("sebie->model.regex_status invalid");
       }
-    } else {
-      sebie_debug("sebie->model.regex_status invalid");
-    }
   }
 }
 
 void
 on_entry_input_path_changed (GtkEditable *editable,
-			     gpointer    user_data)
+                 gpointer    user_data)
 {
   struct sebie_s *sebie = (struct sebie_s *)user_data;
   const gchar *input_path = gtk_entry_get_text(GTK_ENTRY(editable));
@@ -78,7 +78,7 @@ on_entry_input_path_changed (GtkEditable *editable,
 
 void
 on_entry_input_regex_changed (GtkEditable *editable,
-			     gpointer    user_data)
+                 gpointer    user_data)
 {
   struct sebie_s *sebie = (struct sebie_s *)user_data;
   const gchar *input_regex = gtk_entry_get_text(GTK_ENTRY(editable));
@@ -91,7 +91,7 @@ on_entry_input_regex_changed (GtkEditable *editable,
 
 void
 on_entry_output_format_changed (GtkEditable *editable,
-				gpointer    user_data)
+                gpointer    user_data)
 {
   struct sebie_s *sebie = (struct sebie_s *)user_data;
   const gchar *output_format = gtk_entry_get_text(GTK_ENTRY(editable));
@@ -104,8 +104,8 @@ on_entry_output_format_changed (GtkEditable *editable,
 
 gint
 window1_delete_event (GtkWidget *widget,
-		      GdkEvent  *event,
-		      gpointer   data)
+              GdkEvent  *event,
+              gpointer   data)
 {
     sebie_debug("window1_delete_event\n");
     return FALSE;
@@ -114,7 +114,7 @@ window1_delete_event (GtkWidget *widget,
 
 void
 window1_destroy (GtkWidget *widget,
-		 gpointer   data)
+         gpointer   data)
 {
     sebie_debug("window1_destroy\n");
     gtk_main_quit ();
@@ -123,25 +123,25 @@ window1_destroy (GtkWidget *widget,
 
 void
 drawingarea1_size_allocate (GtkWidget *widget,
-			    GtkAllocation *allocation,
-			    gpointer   user_data)
+                GtkAllocation *allocation,
+                gpointer   user_data)
 {
     struct sebie_s *sebie = (struct sebie_s *)user_data;
     gint window1_width;
     gint window1_height;
 
     sebie_debug("drawingarea1_size_allocate x = %d y = %d width = %d height = %d\n",
-		allocation->x,
-		allocation->y,
-		allocation->width,
-		allocation->height);
- 
+        allocation->x,
+        allocation->y,
+        allocation->width,
+        allocation->height);
+
     gtk_window_get_size(GTK_WINDOW(sebie->view.window1), &window1_width, &window1_height);
     sebie->model.window1_width = window1_width;
     sebie->model.window1_height = window1_height;
     sebie->model.pixbuf_width = allocation->width;
     sebie->model.pixbuf_height = allocation->height;
- 
+
     sebie->model_tags |= SEBIE_MODEL_CHANGE_PIXBUF_SIZE;
     save_sebie_config(sebie);
 }
@@ -149,7 +149,7 @@ drawingarea1_size_allocate (GtkWidget *widget,
 
 void
 on_button_quit_clicked (GtkButton *button,
-			gpointer  user_data)
+            gpointer  user_data)
 {
   gtk_main_quit ();
 }
@@ -157,7 +157,7 @@ on_button_quit_clicked (GtkButton *button,
 
 void
 on_button_save_clicked (GtkButton *button,
-			gpointer  user_data)
+            gpointer  user_data)
 {
   struct sebie_s *sebie = (struct sebie_s *)user_data;
 
@@ -169,13 +169,13 @@ on_button_save_clicked (GtkButton *button,
   sebie_debug("selection_width:  %d\n", sebie->model.selection_width);
   sebie_debug("selection_height: %d\n", sebie->model.selection_height);
   sebie_debug("scaled_scale: %lf\n", sebie->model.scaled_scale);
-  
-  if (sebie->model.selection_width == 0 || 
+
+  if (sebie->model.selection_width == 0 ||
       sebie->model.selection_height == 0) {
     sebie_error_popup(sebie, "No region selected.");
     return;
   }
-  
+
   int x = (double)sebie->model.selection_x / sebie->model.scaled_scale;
   int y = (double)sebie->model.selection_y / sebie->model.scaled_scale;
   int width = (double)sebie->model.selection_width / sebie->model.scaled_scale;
@@ -194,7 +194,7 @@ on_button_save_clicked (GtkButton *button,
     gint tmp2 = width;
     x = y;
     width = height;
-    height = tmp2;   
+    height = tmp2;
     y = sebie->model.image_height - tmp1 - tmp2;
     fac_width = sebie->model.selection_fac_height;
     fac_height = sebie->model.selection_fac_width;
@@ -213,7 +213,7 @@ on_button_save_clicked (GtkButton *button,
   }
   sebie_debug("without rotation:\n");
   sebie_debug("x:  %d\n", x);
-  sebie_debug("y:  %d\n", y); 
+  sebie_debug("y:  %d\n", y);
   sebie_debug("width:  %d\n", width);
   sebie_debug("height: %d\n", height);
 
@@ -223,24 +223,24 @@ on_button_save_clicked (GtkButton *button,
     sebie_error_popup(sebie, "Selected region out of range.");
     return;
   }
-  
+
   GdkPixbuf *sub = gdk_pixbuf_new_subpixbuf(sebie->model.image_pixbuf,
-					    x, y,
-					    width, height);
+                        x, y,
+                        width, height);
 
   GdkPixbuf *corr = pixbuf_color_corrections(sub,
-					     sebie->model.image_gamma,
-					     sebie->model.image_grayscale);
+                         sebie->model.image_gamma,
+                         sebie->model.image_grayscale);
 
-  GdkPixbuf *save = gdk_pixbuf_scale_simple(corr, 
-					    fac_width,
-					    fac_height,
-					    GDK_INTERP_HYPER);
+  GdkPixbuf *save = gdk_pixbuf_scale_simple(corr,
+                        fac_width,
+                        fac_height,
+                        GDK_INTERP_HYPER);
 
   if (corr != NULL) {
     g_object_unref(corr);
   }
-  
+
   if (sebie->model.scaled_rotation == SEBIE_ROTATE_090 ||
       sebie->model.scaled_rotation == SEBIE_ROTATE_180 ||
       sebie->model.scaled_rotation == SEBIE_ROTATE_270 ) {
@@ -251,7 +251,7 @@ on_button_save_clicked (GtkButton *button,
     save = rotated;
   }
   GError *error = NULL;
-  
+
   if (gdk_pixbuf_save(save, sebie->model.output_file_name, "jpeg", &error, "quality", "85", NULL) == FALSE) {
     sebie_error_popup(sebie, "gdb_pixbuf_save failed.");
   }
@@ -267,25 +267,37 @@ on_button_save_clicked (GtkButton *button,
 
 void
 on_button_rotate_left_clicked (GtkButton *button,
-			       gpointer  user_data)
+                   gpointer  user_data)
 {
   struct sebie_s *sebie = (struct sebie_s *)user_data;
+
+  // no image loaded yet, simply do nothing
+  if (sebie->model.scaled_pixbuf == NULL) {
+    return;
+  }
+
   sebie->model.scaled_rotation = (sebie->model.scaled_rotation + SEBIE_ROTATE_270) % SEBIE_ROTATE_360;
   rotate_scaled(sebie, SEBIE_ROTATE_270);
 }
 
 void
 on_button_rotate_right_clicked (GtkButton *button,
-				gpointer  user_data)
+                gpointer  user_data)
 {
   struct sebie_s *sebie = (struct sebie_s *)user_data;
+
+  // no image loaded yet, simply do nothing
+  if (sebie->model.scaled_pixbuf == NULL) {
+    return;
+  }
+
   sebie->model.scaled_rotation = (sebie->model.scaled_rotation + SEBIE_ROTATE_090) % SEBIE_ROTATE_360;
   rotate_scaled(sebie, SEBIE_ROTATE_090);
 }
 
 void
 on_entry_selection_fac_width_changed (GtkEditable *editable,
-				      gpointer  user_data)
+                      gpointer  user_data)
 {
   struct sebie_s *sebie = (struct sebie_s *)user_data;
   if (sebie->dirty_tags == 0) {
@@ -300,7 +312,7 @@ on_entry_selection_fac_width_changed (GtkEditable *editable,
 
 void
 on_entry_selection_fac_height_changed (GtkEditable *editable,
-				       gpointer  user_data)
+                       gpointer  user_data)
 {
   struct sebie_s *sebie = (struct sebie_s *)user_data;
   if (sebie->dirty_tags == 0) {
@@ -315,7 +327,7 @@ on_entry_selection_fac_height_changed (GtkEditable *editable,
 
 void
 on_button_fac_swap (GtkButton *button,
-		    gpointer  user_data)
+            gpointer  user_data)
 {
   struct sebie_s *sebie = (struct sebie_s *)user_data;
   gint tmp = sebie->model.selection_fac_width;
@@ -330,7 +342,7 @@ on_button_fac_swap (GtkButton *button,
 
 void
 on_entry_selection_gamma_changed (GtkEditable *editable,
-				  gpointer  user_data)
+                  gpointer  user_data)
 {
   struct sebie_s *sebie = (struct sebie_s *)user_data;
   const gchar *gamma_str = gtk_entry_get_text(GTK_ENTRY(editable));
@@ -355,9 +367,21 @@ on_entry_selection_gamma_changed (GtkEditable *editable,
   apply_color_corrections(sebie);
 }
 
+#if GTK_CHECK_VERSION(3,0,0)
+void
+on_button_grayscale_clicked(GtkToolItem *button,
+                gpointer  user_data)
+{
+  struct sebie_s *sebie = (struct sebie_s *)user_data;
+  sebie->model.image_grayscale = gtk_toggle_tool_button_get_active (GTK_TOGGLE_TOOL_BUTTON(button));
+  sebie_debug("set new image_grayscale = %d\n", sebie->model.image_grayscale);
+  sebie->model_tags |= SEBIE_MODEL_UPDATE_GRAYSCALE;
+  apply_color_corrections(sebie);
+}
+#else
 void
 on_button_grayscale_clicked(GtkToggleButton *button,
-			    gpointer  user_data)
+                gpointer  user_data)
 {
   struct sebie_s *sebie = (struct sebie_s *)user_data;
   sebie->model.image_grayscale = gtk_toggle_button_get_active (button);
@@ -365,13 +389,17 @@ on_button_grayscale_clicked(GtkToggleButton *button,
   sebie->model_tags |= SEBIE_MODEL_UPDATE_GRAYSCALE;
   apply_color_corrections(sebie);
 }
+#endif
 
 gboolean
 on_drawingarea1_expose_event (GtkWidget       *widget,
-			      GdkEventExpose  *event,
-			      gpointer         user_data)
+                  GdkEventExpose  *event,
+                  gpointer         user_data)
 {
   struct sebie_s *sebie = (struct sebie_s *)user_data;
+#if GTK_CHECK_VERSION(3,0,0)
+  sebie->dirty_tags |= SEBIE_UPDATE_SCALED_PIXBUF;
+#endif
   sebie->dirty_tags |= SEBIE_UPDATE_SELECTION_RECT;
   sebie_update_view(sebie);
   return FALSE;
@@ -382,7 +410,7 @@ in_selected_area(gint x, gint y, struct sebie_s *sebie)
 {
   if (x >= sebie->model.selection_x &&
       x <= sebie->model.selection_x + sebie->model.selection_width &&
-      y >= sebie->model.selection_y && 
+      y >= sebie->model.selection_y &&
       y <= sebie->model.selection_y + sebie->model.selection_height) {
     return TRUE;
   } else {
@@ -444,8 +472,8 @@ in_lower_left_corner(gint x, gint y, struct sebie_s *sebie)
 
 gboolean
 on_drawingarea1_button_press_event (GtkWidget       *widget,
-				    GdkEventButton  *event,
-				    gpointer         user_data)
+                    GdkEventButton  *event,
+                    gpointer         user_data)
 {
   struct sebie_s *sebie = (struct sebie_s *)user_data;
 
@@ -481,27 +509,37 @@ on_drawingarea1_button_press_event (GtkWidget       *widget,
       sebie->model.selection_width  = 0;
       sebie->model.selection_height = 0;
     }
-    
+
     sebie->dirty_tags |= SEBIE_UPDATE_SELECTION_RECT;
     sebie_update_view(sebie);
   }
-  
+
   return FALSE;
 }
 
 gboolean
 on_drawingarea1_motion_notify_event (GtkWidget       *widget,
-				     GdkEventMotion  *event,
-				     gpointer         user_data)
+                     GdkEventMotion  *event,
+                     gpointer         user_data)
 {
   struct sebie_s *sebie = (struct sebie_s *)user_data;
-  
+
   gint x;
   gint y;
   GdkModifierType state;
-  
+
+#if GTK_CHECK_VERSION(3,0,0)
+  GdkDisplay* display = gdk_display_get_default();
+  GdkSeat* seat = gdk_display_get_default_seat(display);
+  GdkDevice* pointer = gdk_seat_get_pointer(seat);
+#endif
+
   if (event->is_hint) {
+#if GTK_CHECK_VERSION(3,0,0)
+    gdk_window_get_device_position(event->window, pointer, &x, &y, &state);
+#else
     gdk_window_get_pointer (event->window, &x, &y, &state);
+#endif
   } else {
     x = event->x;
     y = event->y;
@@ -518,72 +556,71 @@ on_drawingarea1_motion_notify_event (GtkWidget       *widget,
       gint y2 = sebie->model.selection_y + sebie->model.selection_height;
 
       if (sebie->model.resize_corner == SEBIE_UPPER_LEFT_CORNER) {
-	gint nwidth  = sebie->model.selection_x + sebie->model.selection_width - x;
-	gint nheight = sebie->model.selection_y + sebie->model.selection_height - y;
-	double nfrac = (double)nwidth / (double)nheight;
-	if (nfrac >= sebie->model.selection_fac) {
-	  sebie->model.selection_x = x2 - (double)nheight * sebie->model.selection_fac;
-	  sebie->model.selection_y = y;
-	  sebie->model.selection_width  = x2 - sebie->model.selection_x;
-	  sebie->model.selection_height = y2 - sebie->model.selection_y;
-	  
-	} else {
-	  sebie->model.selection_x = x;
-	  sebie->model.selection_y = y2 - (double)nwidth / sebie->model.selection_fac;
-	  sebie->model.selection_width = x2 - x;
-	  sebie->model.selection_height = y2 - sebie->model.selection_y;
-	}
+        gint nwidth  = sebie->model.selection_x + sebie->model.selection_width - x;
+        gint nheight = sebie->model.selection_y + sebie->model.selection_height - y;
+        double nfrac = (double)nwidth / (double)nheight;
+        if (nfrac >= sebie->model.selection_fac) {
+          sebie->model.selection_x = x2 - (double)nheight * sebie->model.selection_fac;
+          sebie->model.selection_y = y;
+          sebie->model.selection_width  = x2 - sebie->model.selection_x;
+          sebie->model.selection_height = y2 - sebie->model.selection_y;
+        } else {
+          sebie->model.selection_x = x;
+          sebie->model.selection_y = y2 - (double)nwidth / sebie->model.selection_fac;
+          sebie->model.selection_width = x2 - x;
+          sebie->model.selection_height = y2 - sebie->model.selection_y;
+        }
       } else if (sebie->model.resize_corner == SEBIE_UPPER_RIGHT_CORNER) {
-	gint nwidth  = x - sebie->model.selection_x;
-	gint nheight = sebie->model.selection_y + sebie->model.selection_height - y;
-	double nfrac = (double)nwidth / (double)nheight;
-	if (nfrac >= sebie->model.selection_fac) {
-	  sebie->model.selection_width = (double)nheight * sebie->model.selection_fac;
-	  sebie->model.selection_y = y;
-	  sebie->model.selection_height = y2 - sebie->model.selection_y;
-	} else {
-	  sebie->model.selection_width = nwidth;
-	  sebie->model.selection_height = (double)nwidth / sebie->model.selection_fac;
-	  sebie->model.selection_y = y2 - sebie->model.selection_height;
-	}
+        gint nwidth  = x - sebie->model.selection_x;
+        gint nheight = sebie->model.selection_y + sebie->model.selection_height - y;
+        double nfrac = (double)nwidth / (double)nheight;
+        if (nfrac >= sebie->model.selection_fac) {
+          sebie->model.selection_width = (double)nheight * sebie->model.selection_fac;
+          sebie->model.selection_y = y;
+          sebie->model.selection_height = y2 - sebie->model.selection_y;
+        } else {
+            sebie->model.selection_width = nwidth;
+            sebie->model.selection_height = (double)nwidth / sebie->model.selection_fac;
+            sebie->model.selection_y = y2 - sebie->model.selection_height;
+        }
       } else if (sebie->model.resize_corner == SEBIE_LOWER_RIGHT_CORNER) {
-	gint nwidth  = x - sebie->model.selection_x;
-	gint nheight = y - sebie->model.selection_y;
-	double nfrac = (double)nwidth / (double)nheight;
-	if (nfrac >= sebie->model.selection_fac) {
-	  sebie->model.selection_width = (double)nheight * sebie->model.selection_fac;
-	  sebie->model.selection_height = nheight;
-	} else {
-	  sebie->model.selection_width = nwidth;
-	  sebie->model.selection_height = (double)nwidth / sebie->model.selection_fac; 
-	}
+        gint nwidth  = x - sebie->model.selection_x;
+        gint nheight = y - sebie->model.selection_y;
+        double nfrac = (double)nwidth / (double)nheight;
+        if (nfrac >= sebie->model.selection_fac) {
+          sebie->model.selection_width = (double)nheight * sebie->model.selection_fac;
+          sebie->model.selection_height = nheight;
+        } else {
+          sebie->model.selection_width = nwidth;
+          sebie->model.selection_height = (double)nwidth / sebie->model.selection_fac;
+        }
       } else { /* SEBIE_LOWER_LEFT_CORNER */
-	gint nwidth  = sebie->model.selection_x + sebie->model.selection_width - x;
-	gint nheight = y - sebie->model.selection_y;
-	double nfrac = (double)nwidth / (double)nheight;
-	if (nfrac >= sebie->model.selection_fac) {
-	  sebie->model.selection_width = (double)nheight * sebie->model.selection_fac;
-	  sebie->model.selection_height = nheight;
-	  sebie->model.selection_x = x2 - sebie->model.selection_width;
-	} else {
-	  sebie->model.selection_x = x;
-	  sebie->model.selection_height = (double)nwidth / sebie->model.selection_fac;
-	  sebie->model.selection_width = x2 - sebie->model.selection_x;
-	}
+        gint nwidth  = sebie->model.selection_x + sebie->model.selection_width - x;
+        gint nheight = y - sebie->model.selection_y;
+        double nfrac = (double)nwidth / (double)nheight;
+        if (nfrac >= sebie->model.selection_fac) {
+          sebie->model.selection_width = (double)nheight * sebie->model.selection_fac;
+          sebie->model.selection_height = nheight;
+          sebie->model.selection_x = x2 - sebie->model.selection_width;
+        } else {
+          sebie->model.selection_x = x;
+          sebie->model.selection_height = (double)nwidth / sebie->model.selection_fac;
+          sebie->model.selection_width = x2 - sebie->model.selection_x;
+        }
       }
       /* adjust if selection crossed the fix point */
       if (sebie->model.selection_width < 0 &&
-	  sebie->model.selection_height < 0) {
-	sebie->model.selection_x += sebie->model.selection_width;
-	sebie->model.selection_width = sebie->model.selection_width * (-1);
-	sebie->model.selection_y += sebie->model.selection_height;
-	sebie->model.selection_height = sebie->model.selection_height * (-1);
-	switch(sebie->model.resize_corner) {
-	  case SEBIE_UPPER_LEFT_CORNER: sebie->model.resize_corner = SEBIE_LOWER_RIGHT_CORNER; break;
-	  case SEBIE_UPPER_RIGHT_CORNER: sebie->model.resize_corner = SEBIE_LOWER_LEFT_CORNER; break;
-	  case SEBIE_LOWER_RIGHT_CORNER: sebie->model.resize_corner = SEBIE_UPPER_LEFT_CORNER; break;
-	  case SEBIE_LOWER_LEFT_CORNER: sebie->model.resize_corner = SEBIE_UPPER_RIGHT_CORNER; break;
-	}
+          sebie->model.selection_height < 0) {
+        sebie->model.selection_x += sebie->model.selection_width;
+        sebie->model.selection_width = sebie->model.selection_width * (-1);
+        sebie->model.selection_y += sebie->model.selection_height;
+        sebie->model.selection_height = sebie->model.selection_height * (-1);
+        switch(sebie->model.resize_corner) {
+          case SEBIE_UPPER_LEFT_CORNER: sebie->model.resize_corner = SEBIE_LOWER_RIGHT_CORNER; break;
+          case SEBIE_UPPER_RIGHT_CORNER: sebie->model.resize_corner = SEBIE_LOWER_LEFT_CORNER; break;
+          case SEBIE_LOWER_RIGHT_CORNER: sebie->model.resize_corner = SEBIE_UPPER_LEFT_CORNER; break;
+          case SEBIE_LOWER_LEFT_CORNER: sebie->model.resize_corner = SEBIE_UPPER_RIGHT_CORNER; break;
+        }
       }
     } else {
       sebie_debug("move selection\n");
@@ -599,27 +636,47 @@ on_drawingarea1_motion_notify_event (GtkWidget       *widget,
   }
 
   if (in_selected_area(x,y, sebie)) {
+#if GTK_CHECK_VERSION(3,0,0)
+    GdkCursor *cursor = gdk_cursor_new_for_display(display, GDK_FLEUR);
+#else
     GdkCursor *cursor = gdk_cursor_new(GDK_FLEUR);
+#endif
     gdk_window_set_cursor(event->window, cursor);
   } else if (in_upper_left_corner(x, y, sebie) ||
-	     ((state & GDK_BUTTON1_MASK) &&
-	      sebie->model.resize_corner == SEBIE_UPPER_LEFT_CORNER)) {
+         ((state & GDK_BUTTON1_MASK) &&
+          sebie->model.resize_corner == SEBIE_UPPER_LEFT_CORNER)) {
+#if GTK_CHECK_VERSION(3,0,0)
+    GdkCursor *cursor = gdk_cursor_new_for_display(display, GDK_TOP_LEFT_CORNER);
+#else
     GdkCursor *cursor = gdk_cursor_new(GDK_TOP_LEFT_CORNER);
+#endif
     gdk_window_set_cursor(event->window, cursor);
   } else if (in_upper_right_corner(x, y, sebie) ||
-	     ((state & GDK_BUTTON1_MASK) &&
-	      sebie->model.resize_corner == SEBIE_UPPER_RIGHT_CORNER)) {
+         ((state & GDK_BUTTON1_MASK) &&
+          sebie->model.resize_corner == SEBIE_UPPER_RIGHT_CORNER)) {
+#if GTK_CHECK_VERSION(3,0,0)
+    GdkCursor *cursor = gdk_cursor_new_for_display(display, GDK_TOP_RIGHT_CORNER);
+#else
     GdkCursor *cursor = gdk_cursor_new(GDK_TOP_RIGHT_CORNER);
+#endif
     gdk_window_set_cursor(event->window, cursor);
   } else if (in_lower_right_corner(x, y, sebie) ||
-	     ((state & GDK_BUTTON1_MASK) &&
-	      sebie->model.resize_corner == SEBIE_LOWER_RIGHT_CORNER)) {
+         ((state & GDK_BUTTON1_MASK) &&
+          sebie->model.resize_corner == SEBIE_LOWER_RIGHT_CORNER)) {
+#if GTK_CHECK_VERSION(3,0,0)
+    GdkCursor *cursor = gdk_cursor_new_for_display(display, GDK_BOTTOM_RIGHT_CORNER);
+#else
     GdkCursor *cursor = gdk_cursor_new(GDK_BOTTOM_RIGHT_CORNER);
+#endif
     gdk_window_set_cursor(event->window, cursor);
   } else if (in_lower_left_corner(x, y, sebie) ||
-	     ((state & GDK_BUTTON1_MASK) &&
-	      sebie->model.resize_corner == SEBIE_LOWER_LEFT_CORNER)) {
+         ((state & GDK_BUTTON1_MASK) &&
+          sebie->model.resize_corner == SEBIE_LOWER_LEFT_CORNER)) {
+#if GTK_CHECK_VERSION(3,0,0)
+    GdkCursor *cursor = gdk_cursor_new_for_display(display, GDK_BOTTOM_LEFT_CORNER);
+#else
     GdkCursor *cursor = gdk_cursor_new(GDK_BOTTOM_LEFT_CORNER);
+#endif
     gdk_window_set_cursor(event->window, cursor);
   } else {
     gdk_window_set_cursor (event->window, NULL);
