@@ -25,7 +25,7 @@
 
 #define GLADE_HOOKUP_OBJECT(component,widget,name) \
   g_object_set_data_full (G_OBJECT (component), name, \
-    gtk_widget_ref (widget), (GDestroyNotify) gtk_widget_unref)
+    g_object_ref (widget), (GDestroyNotify) g_object_unref)
 
 #define GLADE_HOOKUP_OBJECT_NO_REF(component,widget,name) \
   g_object_set_data (G_OBJECT (component), name, widget)
@@ -36,16 +36,23 @@ init_sebie_view(struct sebie_s *sebie)
 {
   GtkWidget *tmp_toolbar_icon;
   GtkAdjustment *gamma_adjustment;
- 
+#if GTK_CHECK_VERSION(3,0,0)
+  GtkToolItem *tmp_tool_item;
+#endif
+
   sebie->view.window1 = gtk_window_new (GTK_WINDOW_TOPLEVEL);
   gchar * window1_title = "SeBIE " PACKAGE_VERSION ;
   gtk_widget_set_name (sebie->view.window1, window1_title);
   gtk_window_set_title (GTK_WINDOW (sebie->view.window1), window1_title);
 
+#if GTK_CHECK_VERSION(3,0,0)
+  sebie->view.vbox1 = gtk_box_new (GTK_ORIENTATION_VERTICAL, 0);
+#else
   sebie->view.vbox1 = gtk_vbox_new (FALSE, 0);
+#endif
   gtk_widget_show (sebie->view.vbox1);
   gtk_container_add (GTK_CONTAINER (sebie->view.window1), sebie->view.vbox1);
-  
+
   sebie->view.toolbar1 = gtk_toolbar_new ();
   gtk_widget_show (sebie->view.toolbar1);
   gtk_box_pack_start (GTK_BOX (sebie->view.vbox1), sebie->view.toolbar1, FALSE, FALSE, 0);
@@ -56,14 +63,19 @@ init_sebie_view(struct sebie_s *sebie)
 #else
   tmp_toolbar_icon = NULL;
 #endif
+#if GTK_CHECK_VERSION(3,0,0)
+  sebie->view.button_quit =  gtk_tool_button_new(tmp_toolbar_icon, _("Quit"));
+  gtk_toolbar_insert (GTK_TOOLBAR (sebie->view.toolbar1), sebie->view.button_quit, -1);
+#else
   sebie->view.button_quit = gtk_toolbar_append_element (GTK_TOOLBAR (sebie->view.toolbar1),
-						   GTK_TOOLBAR_CHILD_BUTTON,
-						   NULL,
-						   _("Quit"),
-						   NULL, NULL,
-						   tmp_toolbar_icon, NULL, NULL);
+                                                        GTK_TOOLBAR_CHILD_BUTTON,
+                                                        NULL,
+                                                        _("Quit"),
+                                                        NULL, NULL,
+                                                        tmp_toolbar_icon, NULL, NULL);
   gtk_label_set_use_underline (GTK_LABEL (((GtkToolbarChild*) (g_list_last (GTK_TOOLBAR (sebie->view.toolbar1)->children)->data))->label), TRUE);
-  gtk_widget_show (sebie->view.button_quit);
+#endif
+  gtk_widget_show (GTK_WIDGET(sebie->view.button_quit));
 
 #ifdef USE_ICON
   tmp_toolbar_icon = gtk_image_new_from_stock ("gtk-open", gtk_toolbar_get_icon_size (GTK_TOOLBAR (sebie->view.toolbar1)));
@@ -87,7 +99,10 @@ init_sebie_view(struct sebie_s *sebie)
 #else
   tmp_toolbar_icon = NULL;
 #endif
-
+#if GTK_CHECK_VERSION(3,0,0)
+  sebie->view.button_save =  gtk_tool_button_new(tmp_toolbar_icon, _("Save"));
+  gtk_toolbar_insert (GTK_TOOLBAR (sebie->view.toolbar1), sebie->view.button_save, -1);
+#else
   sebie->view.button_save = gtk_toolbar_append_element (GTK_TOOLBAR (sebie->view.toolbar1),
                                 GTK_TOOLBAR_CHILD_BUTTON,
                                 NULL,
@@ -95,29 +110,37 @@ init_sebie_view(struct sebie_s *sebie)
                                 NULL, NULL,
                                 tmp_toolbar_icon, NULL, NULL);
   gtk_label_set_use_underline (GTK_LABEL (((GtkToolbarChild*) (g_list_last (GTK_TOOLBAR (sebie->view.toolbar1)->children)->data))->label), TRUE);
-  gtk_widget_show (sebie->view.button_save);
+#endif
+  gtk_widget_show (GTK_WIDGET(sebie->view.button_save));
 
 #ifdef USE_ICON
   tmp_toolbar_icon = gtk_image_new_from_stock ("gtk-go-back", gtk_toolbar_get_icon_size (GTK_TOOLBAR (sebie->view.toolbar1)));
 #else
   tmp_toolbar_icon = NULL;
 #endif
-
+#if GTK_CHECK_VERSION(3,0,0)
+  sebie->view.button_rotate_left =  gtk_tool_button_new(tmp_toolbar_icon, _("<- Rotate "));
+  gtk_toolbar_insert (GTK_TOOLBAR (sebie->view.toolbar1), sebie->view.button_rotate_left, -1);
+#else
   sebie->view.button_rotate_left = gtk_toolbar_append_element (GTK_TOOLBAR (sebie->view.toolbar1),
                                 GTK_TOOLBAR_CHILD_BUTTON,
                                 NULL,
-                                _("<- Rotate "),
+                                _("<- Rotate"),
                                 NULL, NULL,
                                 tmp_toolbar_icon, NULL, NULL);
   gtk_label_set_use_underline (GTK_LABEL (((GtkToolbarChild*) (g_list_last (GTK_TOOLBAR (sebie->view.toolbar1)->children)->data))->label), TRUE);
-  gtk_widget_show (sebie->view.button_rotate_left);
+#endif
+  gtk_widget_show (GTK_WIDGET(sebie->view.button_rotate_left));
 
 #ifdef USE_ICON
   tmp_toolbar_icon = gtk_image_new_from_stock ("gtk-go-forward", gtk_toolbar_get_icon_size (GTK_TOOLBAR (sebie->view.toolbar1)));
 #else
   tmp_toolbar_icon = NULL;
 #endif
-
+#if GTK_CHECK_VERSION(3,0,0)
+  sebie->view.button_rotate_right =  gtk_tool_button_new(tmp_toolbar_icon, _("Rotate ->"));
+  gtk_toolbar_insert (GTK_TOOLBAR (sebie->view.toolbar1), sebie->view.button_rotate_right, -1);
+#else
   sebie->view.button_rotate_right = gtk_toolbar_append_element (GTK_TOOLBAR (sebie->view.toolbar1),
                                 GTK_TOOLBAR_CHILD_BUTTON,
                                 NULL,
@@ -125,27 +148,56 @@ init_sebie_view(struct sebie_s *sebie)
                                 NULL, NULL,
                                 tmp_toolbar_icon, NULL, NULL);
   gtk_label_set_use_underline (GTK_LABEL (((GtkToolbarChild*) (g_list_last (GTK_TOOLBAR (sebie->view.toolbar1)->children)->data))->label), TRUE);
-  gtk_widget_show (sebie->view.button_rotate_right);
+#endif
+  gtk_widget_show (GTK_WIDGET(sebie->view.button_rotate_right));
 
   sebie->view.label_selection_fac_width = gtk_label_new (_(" w:"));
   gtk_widget_show (sebie->view.label_selection_fac_width);
+
+#if GTK_CHECK_VERSION(3,0,0)
+  tmp_tool_item = gtk_tool_item_new();
+  gtk_container_add(GTK_CONTAINER(tmp_tool_item), sebie->view.label_selection_fac_width);
+  gtk_toolbar_insert (GTK_TOOLBAR (sebie->view.toolbar1), tmp_tool_item, -1);
+  gtk_widget_show(GTK_WIDGET(tmp_tool_item));
+#else
   gtk_toolbar_append_widget (GTK_TOOLBAR (sebie->view.toolbar1), sebie->view.label_selection_fac_width, NULL, NULL);
+#endif
   gtk_label_set_justify (GTK_LABEL (sebie->view.label_selection_fac_width), GTK_JUSTIFY_LEFT);
 
   sebie->view.entry_selection_fac_width = gtk_entry_new ();
   gtk_widget_show (sebie->view.entry_selection_fac_width);
+#if GTK_CHECK_VERSION(3,0,0)
+  tmp_tool_item = gtk_tool_item_new();
+  gtk_container_add(GTK_CONTAINER(tmp_tool_item), sebie->view.entry_selection_fac_width);
+  gtk_toolbar_insert (GTK_TOOLBAR (sebie->view.toolbar1), tmp_tool_item, -1);
+  gtk_widget_show(GTK_WIDGET(tmp_tool_item));
+#else
   gtk_toolbar_append_widget (GTK_TOOLBAR (sebie->view.toolbar1), sebie->view.entry_selection_fac_width, NULL, NULL);
+#endif
   gtk_widget_set_size_request (sebie->view.entry_selection_fac_width, 50, 20);
   gtk_entry_set_max_length (GTK_ENTRY (sebie->view.entry_selection_fac_width), 5);
-  
   sebie->view.label_selection_fac_height = gtk_label_new (_(" h:"));
   gtk_widget_show (sebie->view.label_selection_fac_height);
+#if GTK_CHECK_VERSION(3,0,0)
+  tmp_tool_item = gtk_tool_item_new();
+  gtk_container_add(GTK_CONTAINER(tmp_tool_item), sebie->view.label_selection_fac_height);
+  gtk_toolbar_insert (GTK_TOOLBAR (sebie->view.toolbar1), tmp_tool_item, -1);
+  gtk_widget_show(GTK_WIDGET(tmp_tool_item));
+#else
   gtk_toolbar_append_widget (GTK_TOOLBAR (sebie->view.toolbar1), sebie->view.label_selection_fac_height, NULL, NULL);
+#endif
   gtk_label_set_justify (GTK_LABEL (sebie->view.label_selection_fac_height), GTK_JUSTIFY_LEFT);
 
   sebie->view.entry_selection_fac_height = gtk_entry_new ();
   gtk_widget_show (sebie->view.entry_selection_fac_height);
+#if GTK_CHECK_VERSION(3,0,0)
+  tmp_tool_item = gtk_tool_item_new();
+  gtk_container_add(GTK_CONTAINER(tmp_tool_item), sebie->view.entry_selection_fac_height);
+  gtk_toolbar_insert (GTK_TOOLBAR (sebie->view.toolbar1), tmp_tool_item, -1);
+  gtk_widget_show(GTK_WIDGET(tmp_tool_item));
+#else
   gtk_toolbar_append_widget (GTK_TOOLBAR (sebie->view.toolbar1), sebie->view.entry_selection_fac_height, NULL, NULL);
+#endif
   gtk_widget_set_size_request (sebie->view.entry_selection_fac_height, 50, 20);
   gtk_entry_set_max_length (GTK_ENTRY (sebie->view.entry_selection_fac_height), 5);
 
@@ -154,6 +206,10 @@ init_sebie_view(struct sebie_s *sebie)
 #else
   tmp_toolbar_icon = NULL;
 #endif
+#if GTK_CHECK_VERSION(3,0,0)
+  sebie->view.button_fac_swap =  gtk_tool_button_new(tmp_toolbar_icon, _("Swap"));
+  gtk_toolbar_insert (GTK_TOOLBAR (sebie->view.toolbar1), sebie->view.button_fac_swap, -1);
+#else
  sebie->view.button_fac_swap = gtk_toolbar_append_element (GTK_TOOLBAR (sebie->view.toolbar1),
                                 GTK_TOOLBAR_CHILD_BUTTON,
                                 NULL,
@@ -161,46 +217,82 @@ init_sebie_view(struct sebie_s *sebie)
                                 NULL, NULL,
                                 tmp_toolbar_icon, NULL, NULL);
   gtk_label_set_use_underline (GTK_LABEL (((GtkToolbarChild*) (g_list_last (GTK_TOOLBAR (sebie->view.toolbar1)->children)->data))->label), TRUE);
-  gtk_widget_show (sebie->view.button_fac_swap);
+#endif
+  gtk_widget_show (GTK_WIDGET(sebie->view.button_fac_swap));
 
   sebie->view.label_selection_fac = gtk_label_new (_("label1"));
   gtk_widget_show (sebie->view.label_selection_fac);
+#if GTK_CHECK_VERSION(3,0,0)
+  tmp_tool_item = gtk_tool_item_new();
+  gtk_container_add(GTK_CONTAINER(tmp_tool_item), sebie->view.label_selection_fac);
+  gtk_toolbar_insert (GTK_TOOLBAR (sebie->view.toolbar1), tmp_tool_item, -1);
+  gtk_widget_show(GTK_WIDGET(tmp_tool_item));
+#else
   gtk_toolbar_append_widget (GTK_TOOLBAR (sebie->view.toolbar1), sebie->view.label_selection_fac, NULL, NULL);
+#endif
   gtk_label_set_justify (GTK_LABEL (sebie->view.label_selection_fac), GTK_JUSTIFY_LEFT);
 
   sebie->view.label_selection_gamma = gtk_label_new (_(" gamma:"));
   gtk_widget_show (sebie->view.label_selection_gamma);
+#if GTK_CHECK_VERSION(3,0,0)
+  tmp_tool_item = gtk_tool_item_new();
+  gtk_container_add(GTK_CONTAINER(tmp_tool_item), sebie->view.label_selection_gamma);
+  gtk_toolbar_insert (GTK_TOOLBAR (sebie->view.toolbar1), tmp_tool_item, -1);
+  gtk_widget_show(GTK_WIDGET(tmp_tool_item));
+#else
   gtk_toolbar_append_widget (GTK_TOOLBAR (sebie->view.toolbar1), sebie->view.label_selection_gamma, NULL, NULL);
+#endif
   gtk_label_set_justify (GTK_LABEL (sebie->view.label_selection_gamma), GTK_JUSTIFY_LEFT);
 
 #if 1
   gamma_adjustment = (GtkAdjustment *)gtk_adjustment_new(1.0, 0.1, 3.0, 0.1, 0.1, 0.0);
   sebie->view.entry_selection_gamma = gtk_spin_button_new(gamma_adjustment,
-							  0.0,
-							  1);
-  
+                                                          0.0,
+                                                          1);
 #else
   sebie->view.entry_selection_gamma = gtk_entry_new ();
 #endif
   gtk_widget_show (sebie->view.entry_selection_gamma);
+
+#if GTK_CHECK_VERSION(3,0,0)
+  tmp_tool_item = gtk_tool_item_new();
+  gtk_container_add(GTK_CONTAINER(tmp_tool_item), sebie->view.entry_selection_gamma);
+  gtk_toolbar_insert (GTK_TOOLBAR (sebie->view.toolbar1), tmp_tool_item, -1);
+  gtk_widget_show(GTK_WIDGET(tmp_tool_item));
+#else
   gtk_toolbar_append_widget (GTK_TOOLBAR (sebie->view.toolbar1), sebie->view.entry_selection_gamma, NULL, NULL);
+#endif
   gtk_widget_set_size_request (sebie->view.entry_selection_gamma, 50, 20);
   gtk_entry_set_max_length (GTK_ENTRY (sebie->view.entry_selection_gamma), 5);
 
+#if GTK_CHECK_VERSION(3,0,0)
+  sebie->view.button_grayscale = gtk_toggle_tool_button_new();
+  gtk_tool_button_set_label(GTK_TOOL_BUTTON(sebie->view.button_grayscale), _("Grayscale"));
+  gtk_toolbar_insert (GTK_TOOLBAR (sebie->view.toolbar1), sebie->view.button_grayscale, -1);
+#else
   sebie->view.button_grayscale = gtk_toolbar_append_element (GTK_TOOLBAR (sebie->view.toolbar1),
-							     GTK_TOOLBAR_CHILD_TOGGLEBUTTON,
-							     NULL,
-							     _("Grayscale"),
-							     NULL, NULL,
-							     NULL, NULL, NULL);
+                                                             GTK_TOOLBAR_CHILD_TOGGLEBUTTON,
+                                                             NULL,
+                                                             _("Grayscale"),
+                                                             NULL, NULL,
+                                                             NULL, NULL, NULL);
   gtk_label_set_use_underline (GTK_LABEL (((GtkToolbarChild*) (g_list_last (GTK_TOOLBAR (sebie->view.toolbar1)->children)->data))->label), TRUE);
-  gtk_widget_show (sebie->view.button_grayscale);
+#endif
+  gtk_widget_show (GTK_WIDGET(sebie->view.button_grayscale));
 
+#if GTK_CHECK_VERSION(3,0,0)
+  sebie->view.hpaned1 = gtk_paned_new (GTK_ORIENTATION_HORIZONTAL);
+#else
   sebie->view.hpaned1 = gtk_hpaned_new ();
+#endif
   gtk_widget_show (sebie->view.hpaned1);
   gtk_box_pack_start (GTK_BOX (sebie->view.vbox1), sebie->view.hpaned1, TRUE, TRUE, 0);
 
+#if GTK_CHECK_VERSION(3,0,0)
+  sebie->view.vbox2 = gtk_box_new (GTK_ORIENTATION_VERTICAL, 0);
+#else
   sebie->view.vbox2 = gtk_vbox_new (FALSE, 0);
+#endif
   gtk_widget_show (sebie->view.vbox2);
   gtk_paned_pack1 (GTK_PANED (sebie->view.hpaned1), sebie->view.vbox2, FALSE, TRUE);
 
@@ -256,19 +348,19 @@ init_sebie_view(struct sebie_s *sebie)
   gtk_paned_pack2 (GTK_PANED (sebie->view.hpaned1), sebie->view.drawingarea1, TRUE, TRUE);
 
   gtk_window_set_default_size(GTK_WINDOW (sebie->view.window1),
-			      sebie->model.window1_width,
-			      sebie->model.window1_height);
+                              sebie->model.window1_width,
+                              sebie->model.window1_height);
 
   sebie->view.statusbar1 = gtk_statusbar_new ();
   gtk_widget_show (sebie->view.statusbar1);
   sebie->view.statusbar1_context_id = gtk_statusbar_get_context_id(GTK_STATUSBAR(sebie->view.statusbar1), "SomeContext");
   gtk_box_pack_start (GTK_BOX (sebie->view.vbox1), sebie->view.statusbar1, FALSE, FALSE, 0);
-  
+
   GtkTreeSelection *select = gtk_tree_view_get_selection(GTK_TREE_VIEW(sebie->view.treeview_file_list));
   gtk_tree_selection_set_mode(select, GTK_SELECTION_SINGLE);
   g_signal_connect(G_OBJECT(select), "changed",
-		   G_CALLBACK(on_treeview_file_list_changed),
-		   sebie);
+                   G_CALLBACK(on_treeview_file_list_changed),
+                   sebie);
   g_signal_connect ((gpointer) sebie->view.entry_input_path, "changed",
                     G_CALLBACK (on_entry_input_path_changed),
                     sebie);
@@ -280,12 +372,12 @@ init_sebie_view(struct sebie_s *sebie)
                     sebie);
 
   g_signal_connect (G_OBJECT (sebie->view.window1), "delete-event",
-		    G_CALLBACK (window1_delete_event), NULL);
+                    G_CALLBACK (window1_delete_event), NULL);
   g_signal_connect (G_OBJECT (sebie->view.window1), "destroy",
-		    G_CALLBACK (window1_destroy), NULL);
+                    G_CALLBACK (window1_destroy), NULL);
 
   g_signal_connect (G_OBJECT (sebie->view.drawingarea1), "size-allocate",
-		    G_CALLBACK (drawingarea1_size_allocate), sebie);
+                    G_CALLBACK (drawingarea1_size_allocate), sebie);
 
   g_signal_connect ((gpointer) sebie->view.button_quit, "clicked",
                     G_CALLBACK (on_button_quit_clicked),
@@ -308,9 +400,15 @@ init_sebie_view(struct sebie_s *sebie)
   g_signal_connect ((gpointer) sebie->view.button_fac_swap, "clicked",
                     G_CALLBACK (on_button_fac_swap),
                     sebie);
+#if GTK_CHECK_VERSION(3,0,0)
+  g_signal_connect ((gpointer) sebie->view.drawingarea1, "draw",
+                    G_CALLBACK (on_drawingarea1_expose_event),
+                    sebie);
+#else
   g_signal_connect ((gpointer) sebie->view.drawingarea1, "expose_event",
                     G_CALLBACK (on_drawingarea1_expose_event),
                     sebie);
+#endif
   g_signal_connect ((gpointer) sebie->view.drawingarea1, "button_press_event",
                     G_CALLBACK (on_drawingarea1_button_press_event),
                     sebie);
@@ -331,11 +429,11 @@ init_sebie_view(struct sebie_s *sebie)
                     sebie);
 
   gtk_widget_set_events (sebie->view.drawingarea1, GDK_EXPOSURE_MASK
-			 | GDK_LEAVE_NOTIFY_MASK
-			 | GDK_BUTTON_PRESS_MASK
-			 | GDK_POINTER_MOTION_MASK
-			 | GDK_POINTER_MOTION_HINT_MASK);
-  
+                         | GDK_LEAVE_NOTIFY_MASK
+                         | GDK_BUTTON_PRESS_MASK
+                         | GDK_POINTER_MOTION_MASK
+                         | GDK_POINTER_MOTION_HINT_MASK);
+
   /* Store pointers to all widgets, for use by lookup_widget(). */
   GLADE_HOOKUP_OBJECT_NO_REF (sebie->view.window1, sebie->view.window1, "window1");
   GLADE_HOOKUP_OBJECT (sebie->view.window1, sebie->view.vbox1, "vbox1");
